@@ -389,7 +389,7 @@ fn generate_ethereum_signature() {
             .unwrap();
     let sig: Signature = "0xe108a7776de6b87183b0690484a74daef44aa6daf907e91abaf7bbfa426ae7706b12e0bd44ef7b0634710d99c2d81087a2f39e075158212343a3b2948ecf33d01c".parse().unwrap();
 
-    assert_eq!(private_key.to_public_key().unwrap(), address);
+    assert_eq!(private_key.to_address(), address);
 
     let generated_sig = private_key.sign_ethereum_msg(&checkpoint);
     assert_eq!(sig, generated_sig)
